# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_pyqtgraph_rev2_1.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(1366, 768)
        font = QtGui.QFont()
        font.setFamily("Calibri Light")
        font.setPointSize(11)
        Form.setFont(font)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.s1 = QtWidgets.QGroupBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(3)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.s1.sizePolicy().hasHeightForWidth())
        self.s1.setSizePolicy(sizePolicy)
        self.s1.setObjectName("s1")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.s1)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.s1_lb_2 = QtWidgets.QLabel(self.s1)
        self.s1_lb_2.setObjectName("s1_lb_2")
        self.gridLayout_2.addWidget(self.s1_lb_2, 1, 0, 1, 1)
        self.s1_lb_1 = QtWidgets.QLabel(self.s1)
        self.s1_lb_1.setObjectName("s1_lb_1")
        self.gridLayout_2.addWidget(self.s1_lb_1, 0, 0, 1, 1)
        self.s1_bt_1 = QtWidgets.QPushButton(self.s1)
        self.s1_bt_1.setObjectName("s1_bt_1")
        self.gridLayout_2.addWidget(self.s1_bt_1, 0, 1, 1, 1)
        self.s1_bt_5 = QtWidgets.QPushButton(self.s1)
        self.s1_bt_5.setObjectName("s1_bt_5")
        self.gridLayout_2.addWidget(self.s1_bt_5, 6, 0, 1, 1)
        self.s1_bt_6 = QtWidgets.QPushButton(self.s1)
        self.s1_bt_6.setObjectName("s1_bt_6")
        self.gridLayout_2.addWidget(self.s1_bt_6, 6, 1, 1, 1)
        self.s1_lb_3 = QtWidgets.QLabel(self.s1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.s1_lb_3.sizePolicy().hasHeightForWidth())
        self.s1_lb_3.setSizePolicy(sizePolicy)
        self.s1_lb_3.setObjectName("s1_lb_3")
        self.gridLayout_2.addWidget(self.s1_lb_3, 2, 0, 1, 2)
        self.s1_bt_2 = QtWidgets.QComboBox(self.s1)
        self.s1_bt_2.setObjectName("s1_bt_2")
        self.gridLayout_2.addWidget(self.s1_bt_2, 1, 1, 1, 1)
        self.s1_lb_4 = QtWidgets.QLabel(self.s1)
        self.s1_lb_4.setObjectName("s1_lb_4")
        self.gridLayout_2.addWidget(self.s1_lb_4, 4, 0, 1, 1)
        self.s1_bt_4 = QtWidgets.QComboBox(self.s1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.s1_bt_4.sizePolicy().hasHeightForWidth())
        self.s1_bt_4.setSizePolicy(sizePolicy)
        self.s1_bt_4.setObjectName("s1_bt_4")
        self.s1_bt_4.addItem("")
        self.s1_bt_4.addItem("")
        self.gridLayout_2.addWidget(self.s1_bt_4, 4, 1, 1, 1)
        self.gridLayout.addWidget(self.s1, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(5)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.s2_lb_1 = QtWidgets.QLabel(self.groupBox_2)
        self.s2_lb_1.setObjectName("s2_lb_1")
        self.gridLayout_3.addWidget(self.s2_lb_1, 0, 0, 1, 1)
        self.s2_di_1 = QtWidgets.QDial(self.groupBox_2)
        self.s2_di_1.setObjectName("s2_di_1")
        self.gridLayout_3.addWidget(self.s2_di_1, 0, 1, 1, 1)
        self.s2_lb_2 = QtWidgets.QLabel(self.groupBox_2)
        self.s2_lb_2.setObjectName("s2_lb_2")
        self.gridLayout_3.addWidget(self.s2_lb_2, 1, 0, 1, 1)
        self.s2_di_2 = QtWidgets.QDial(self.groupBox_2)
        self.s2_di_2.setObjectName("s2_di_2")
        self.gridLayout_3.addWidget(self.s2_di_2, 1, 1, 1, 1)
        self.s2_bt_3 = QtWidgets.QPushButton(self.groupBox_2)
        self.s2_bt_3.setObjectName("s2_bt_3")
        self.gridLayout_3.addWidget(self.s2_bt_3, 2, 1, 1, 1)
        self.s2_lb_3 = QtWidgets.QLabel(self.groupBox_2)
        self.s2_lb_3.setObjectName("s2_lb_3")
        self.gridLayout_3.addWidget(self.s2_lb_3, 3, 0, 1, 2)
        self.gridLayout.addWidget(self.groupBox_2, 1, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.s3_lb_1 = QtWidgets.QLabel(self.groupBox_3)
        self.s3_lb_1.setObjectName("s3_lb_1")
        self.gridLayout_4.addWidget(self.s3_lb_1, 0, 0, 1, 1)
        self.s3_line_1 = QtWidgets.QLineEdit(self.groupBox_3)
        self.s3_line_1.setObjectName("s3_line_1")
        self.gridLayout_4.addWidget(self.s3_line_1, 0, 1, 1, 1)
        self.s3_lb_2 = QtWidgets.QLabel(self.groupBox_3)
        self.s3_lb_2.setObjectName("s3_lb_2")
        self.gridLayout_4.addWidget(self.s3_lb_2, 1, 0, 1, 1)
        self.s3_line_2 = QtWidgets.QLineEdit(self.groupBox_3)
        self.s3_line_2.setObjectName("s3_line_2")
        self.gridLayout_4.addWidget(self.s3_line_2, 1, 1, 1, 1)
        self.s3_lb_3 = QtWidgets.QLabel(self.groupBox_3)
        self.s3_lb_3.setObjectName("s3_lb_3")
        self.gridLayout_4.addWidget(self.s3_lb_3, 2, 0, 1, 1)
        self.s3_line_3 = QtWidgets.QLineEdit(self.groupBox_3)
        self.s3_line_3.setObjectName("s3_line_3")
        self.gridLayout_4.addWidget(self.s3_line_3, 2, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox_3, 2, 0, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(5)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.s4_lb_1 = QtWidgets.QLabel(self.groupBox_4)
        self.s4_lb_1.setObjectName("s4_lb_1")
        self.gridLayout_6.addWidget(self.s4_lb_1, 0, 1, 1, 1)
        self.s4_bt_1 = QtWidgets.QPushButton(self.groupBox_4)
        self.s4_bt_1.setObjectName("s4_bt_1")
        self.gridLayout_6.addWidget(self.s4_bt_1, 2, 3, 1, 1)
        self.s4_bt_3 = QtWidgets.QPushButton(self.groupBox_4)
        self.s4_bt_3.setObjectName("s4_bt_3")
        self.gridLayout_6.addWidget(self.s4_bt_3, 3, 5, 1, 1)
        self.s4_line_2 = QtWidgets.QLCDNumber(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(4)
        sizePolicy.setHeightForWidth(self.s4_line_2.sizePolicy().hasHeightForWidth())
        self.s4_line_2.setSizePolicy(sizePolicy)
        self.s4_line_2.setMinimumSize(QtCore.QSize(0, 3))
        self.s4_line_2.setObjectName("s4_line_2")
        self.gridLayout_6.addWidget(self.s4_line_2, 2, 2, 1, 1)
        self.s4_line_1 = QtWidgets.QLCDNumber(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.s4_line_1.sizePolicy().hasHeightForWidth())
        self.s4_line_1.setSizePolicy(sizePolicy)
        self.s4_line_1.setObjectName("s4_line_1")
        self.gridLayout_6.addWidget(self.s4_line_1, 2, 1, 1, 1)
        self.s4_lb_2 = QtWidgets.QLabel(self.groupBox_4)
        self.s4_lb_2.setObjectName("s4_lb_2")
        self.gridLayout_6.addWidget(self.s4_lb_2, 0, 2, 1, 1)
        self.s4_line_3 = QtWidgets.QLineEdit(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.s4_line_3.sizePolicy().hasHeightForWidth())
        self.s4_line_3.setSizePolicy(sizePolicy)
        self.s4_line_3.setObjectName("s4_line_3")
        self.gridLayout_6.addWidget(self.s4_line_3, 3, 2, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_6.addItem(spacerItem, 3, 3, 1, 1)
        self.s4_bt_2 = QtWidgets.QPushButton(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.s4_bt_2.sizePolicy().hasHeightForWidth())
        self.s4_bt_2.setSizePolicy(sizePolicy)
        self.s4_bt_2.setObjectName("s4_bt_2")
        self.gridLayout_6.addWidget(self.s4_bt_2, 2, 5, 1, 1)
        self.s4_lb_3 = QtWidgets.QLabel(self.groupBox_4)
        self.s4_lb_3.setObjectName("s4_lb_3")
        self.gridLayout_6.addWidget(self.s4_lb_3, 3, 1, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 2, 1, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(9)
        sizePolicy.setVerticalStretch(9)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setObjectName("tabWidget")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.lb_photo = QtWidgets.QLabel(self.tab)
        self.lb_photo.setText("")
        self.lb_photo.setObjectName("lb_photo")
        self.gridLayout_7.addWidget(self.lb_photo, 0, 0, 1, 5)
        self.bt_cam = QtWidgets.QComboBox(self.tab)
        self.bt_cam.setObjectName("bt_cam")
        self.bt_cam.addItem("")
        self.gridLayout_7.addWidget(self.bt_cam, 1, 1, 1, 1)
        self.lb_select_cam = QtWidgets.QLabel(self.tab)
        self.lb_select_cam.setObjectName("lb_select_cam")
        self.gridLayout_7.addWidget(self.lb_select_cam, 1, 0, 1, 1)
        self.bt_cam_start = QtWidgets.QPushButton(self.tab)
        self.bt_cam_start.setObjectName("bt_cam_start")
        self.gridLayout_7.addWidget(self.bt_cam_start, 1, 3, 1, 1)
        self.bt_cam_stop = QtWidgets.QPushButton(self.tab)
        self.bt_cam_stop.setObjectName("bt_cam_stop")
        self.gridLayout_7.addWidget(self.bt_cam_stop, 1, 4, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(300, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_7.addItem(spacerItem1, 1, 2, 1, 1)
        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.lb_thers_set = QtWidgets.QLabel(self.tab_2)
        self.lb_thers_set.setObjectName("lb_thers_set")
        self.gridLayout_8.addWidget(self.lb_thers_set, 1, 0, 1, 1)
        self.bt_thers_set = QtWidgets.QComboBox(self.tab_2)
        self.bt_thers_set.setObjectName("bt_thers_set")
        self.bt_thers_set.addItem("")
        self.bt_thers_set.addItem("")
        self.bt_thers_set.addItem("")
        self.bt_thers_set.addItem("")
        self.bt_thers_set.addItem("")
        self.bt_thers_set.addItem("")
        self.gridLayout_8.addWidget(self.bt_thers_set, 1, 1, 1, 1)
        self.bt_start_draw = QtWidgets.QPushButton(self.tab_2)
        self.bt_start_draw.setObjectName("bt_start_draw")
        self.gridLayout_8.addWidget(self.bt_start_draw, 1, 3, 1, 1)
        self.bt_stop_draw = QtWidgets.QPushButton(self.tab_2)
        self.bt_stop_draw.setObjectName("bt_stop_draw")
        self.gridLayout_8.addWidget(self.bt_stop_draw, 1, 4, 1, 1)
        self.bt_save_data = QtWidgets.QPushButton(self.tab_2)
        self.bt_save_data.setObjectName("bt_save_data")
        self.gridLayout_8.addWidget(self.bt_save_data, 1, 5, 1, 1)
        self.plot_widget = GraphicsLayoutWidget(self.tab_2)
        self.plot_widget.setObjectName("plot_widget")
        self.gridLayout_8.addWidget(self.plot_widget, 0, 0, 1, 6)
        spacerItem2 = QtWidgets.QSpacerItem(300, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_8.addItem(spacerItem2, 1, 2, 1, 1)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_3)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.raw_broswer = QtWidgets.QTextBrowser(self.tab_3)
        self.raw_broswer.setObjectName("raw_broswer")
        self.gridLayout_5.addWidget(self.raw_broswer, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_3, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 1, 2, 1)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.s1.setTitle(_translate("Form", "裝置設定"))
        self.s1_lb_2.setText(_translate("Form", "USB serial"))
        self.s1_lb_1.setText(_translate("Form", "檢查裝置"))
        self.s1_bt_1.setText(_translate("Form", "檢查裝置"))
        self.s1_bt_5.setText(_translate("Form", "start"))
        self.s1_bt_6.setText(_translate("Form", "pause"))
        self.s1_lb_3.setText(_translate("Form", "當前裝置"))
        self.s1_lb_4.setText(_translate("Form", "裝置類型"))
        self.s1_bt_4.setItemText(0, _translate("Form", "XMC1302"))
        self.s1_bt_4.setItemText(1, _translate("Form", "microbit"))
        self.groupBox_2.setTitle(_translate("Form", "回控設定"))
        self.s2_lb_1.setText(_translate("Form", "最小閥值"))
        self.s2_lb_2.setText(_translate("Form", "最大閥值"))
        self.s2_bt_3.setText(_translate("Form", "send"))
        self.s2_lb_3.setText(_translate("Form", "設置參數"))
        self.groupBox_3.setTitle(_translate("Form", "系統資訊"))
        self.s3_lb_1.setText(_translate("Form", "開啟時間"))
        self.s3_lb_2.setText(_translate("Form", "當前時間"))
        self.s3_lb_3.setText(_translate("Form", "使用者"))
        self.groupBox_4.setTitle(_translate("Form", "狀態顯示"))
        self.s4_lb_1.setText(_translate("Form", "當前速度"))
        self.s4_bt_1.setText(_translate("Form", "紀錄速度"))
        self.s4_bt_3.setText(_translate("Form", "存取紀錄"))
        self.s4_lb_2.setText(_translate("Form", "Duppler Freq"))
        self.s4_bt_2.setText(_translate("Form", "暫停紀錄"))
        self.s4_lb_3.setText(_translate("Form", "當前狀態"))
        self.bt_cam.setItemText(0, _translate("Form", "默認攝影機(0)"))
        self.lb_select_cam.setText(_translate("Form", "選擇攝影機"))
        self.bt_cam_start.setText(_translate("Form", "開啟攝影"))
        self.bt_cam_stop.setText(_translate("Form", "暫停攝影"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("Form", "攝影機資料"))
        self.lb_thers_set.setText(_translate("Form", "設置參考線"))
        self.bt_thers_set.setItemText(0, _translate("Form", "250"))
        self.bt_thers_set.setItemText(1, _translate("Form", "500"))
        self.bt_thers_set.setItemText(2, _translate("Form", "1000"))
        self.bt_thers_set.setItemText(3, _translate("Form", "1500"))
        self.bt_thers_set.setItemText(4, _translate("Form", "2000"))
        self.bt_thers_set.setItemText(5, _translate("Form", "2500"))
        self.bt_start_draw.setText(_translate("Form", "開啟繪製"))
        self.bt_stop_draw.setText(_translate("Form", "暫停繪製"))
        self.bt_save_data.setText(_translate("Form", "存取當前資料"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("Form", "原始圖表"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_3), _translate("Form", "原始輸入"))


from pyqtgraph import GraphicsLayoutWidget





from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

import pyqtgraph as pg
from  .ui_pyqtgraph_rev2_1 import  Ui_Form
import numpy as np
import serial
import serial.tools.list_ports
import sys
import datetime
#import cv2
import re
import csv

i_raw_line = np.repeat(0, 128)
x_raw_line = np.repeat(100, 128)

class MainWindow(QWidget, Ui_Form):
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)
        self.init_plot()


        self.setupUi(self)
        self.one_plot()
        self.setWindowTitle("Wanshih IC test Graphic User Interface")
        self.setWindowIcon(QIcon("./image/wanshih_logo.png"))

        # uart port
        self.ser = serial.Serial()
        self.port_check()
        # receive data initial
        self.serial_data = 0

        # monitor
        #self.camera_num = cv2.VideoCapture(self.bt_cam.currentIndex())
        self.capture_frame = 0
        self.capture_flag = 0

        #draw plot
        self.i_raw_line = i_raw_line
        self.x_raw_line = x_raw_line
        self.speedRecordDict ={}

        self.initialize()

    def init_plot(self):
        pg.setConfigOption('background', '#f0f0f0')  # 设置背景为灰色
        pg.setConfigOption('foreground', 'd')  # 设置前景（包括坐标轴，线条，文本等等）为黑色。
        pg.setConfigOptions(antialias=True)  # 使曲线看起来更光滑，而不是锯齿状

    def one_plot(self):
        self.plot_widget.clear()

        example_plot = self.plot_widget.addPlot(title = 'XMC1302 Radar Raw data')

        global i_raw_line, x_raw_line
        y1 = i_raw_line
        y2 = x_raw_line
        self.threshold_line = np.repeat(int(self.bt_thers_set.currentText()), 128)

        example_plot.plot(y=y1, pen=(128, 128, 0))
        example_plot.plot(y=y2, pen= (128,0,0))
        example_plot.plot(y=self.threshold_line, pen=(56, 32, 22))

    def initialize(self):
        self.start_time_stamp()
        self.real_time_stamp()

        self.recv_timer = QTimer(self)
        self.recv_timer.timeout.connect(self.data_receive)
        self.analyze_timer = QTimer(self)
        self.analyze_timer.timeout.connect(self.analyze_data)

        #self.monitor_timer = QTimer(self)
        #self.monitor_timer.timeout.connect(self.monitor_check)
        self.plot_timer = QTimer(self)
        self.plot_timer.timeout.connect(self.one_plot)

        self.speed_record_timer = QTimer(self)
        self.speed_record_timer.timeout.connect(self.update_speed_record_dict)

        self.bt_cam_start.clicked.connect(self.open_cam)
        self.bt_cam_stop.clicked.connect(self.stop_cam)

        self.s1_bt_1.clicked.connect(self.port_check)
        self.s1_bt_2.currentTextChanged.connect(self.port_imf)

        self.s1_bt_5.clicked.connect(self.port_open)
        self.s1_bt_6.clicked.connect(self.port_close)

        self.s2_bt_3.clicked.connect(self.data_send)

        self.bt_start_draw.clicked.connect(self.start_draw)
        self.bt_stop_draw.clicked.connect(self.pause_draw)

        self.bt_save_data.clicked.connect(self.save_raw_data)

        self.s4_bt_3.clicked.connect(self.record_speed)
        self.s4_bt_1.clicked.connect(self.start_record)
        self.s4_bt_2.clicked.connect(self.stop_record)

        self.s4_bt_2.setDisabled(True)
        self.s4_bt_3.setDisabled(True)


    def real_time_stamp(self):
        self.sys_timer = QTimer(self)
        self.sys_timer.timeout.connect(self.real_time_update)
        self.sys_timer.start(1000)

    def real_time_update(self):
        time_format = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.s3_line_2.setText(str(time_format))

    def start_time_stamp(self):
        time_format = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        self.s3_line_1.setText(str(time_format))

    '''
        def monitor_check(self):
        ret, frame = self.camera_num.read()
        if (1 == ret):
            file_name = 'capture.jpeg'
            cv2.imwrite(file_name, frame)
            pixmap = QPixmap("./capture.jpeg")

            try:
                if(1 == self.capture_flag):
                    pixmap.fill(color=Qt.red)
                    self.capture_flag = 0
                elif(2 == self.capture_flag):
                    pixmap.fill(color=Qt.darkCyan)
                    self.capture_flag = 0
            except:
                return None

        self.lb_photo.setPixmap(pixmap)
    '''


    def port_check(self):
        # 检测所有存在的串口，将信息存储在字典中
        self.Com_Dict = {}
        port_list = list(serial.tools.list_ports.comports())
        self.s1_bt_2.clear()
        for port in port_list:
            self.Com_Dict["%s" % port[0]] = "%s" % port[1]
            self.s1_bt_2.addItem(port[0])
        if len(self.Com_Dict) == 0:
            self.s1_lb_3.setText(" 无串口")

    def port_imf(self):
        s_imf = self.s1_bt_2.currentText()
        if s_imf != "":
            self.s1_lb_3.setText(self.Com_Dict[self.s1_bt_2.currentText()])

    def port_open(self):
        self.ser.port = self.s1_bt_2.currentText()
        self.ser.baudrate = 128000
        self.ser.bytesize = 8
        self.ser.stopbits = 1
        self.ser.parity = "N"

        try:
            self.ser.open()
        except:
            QMessageBox.critical(self, "Port Error", "此裝置無法開啟！")
            return None

        # 打开串口接收定时器，周期为2ms
        self.recv_timer.start(300)
        self.analyze_timer.start(300)

        if self.ser.isOpen():
            self.s1_bt_5.setEnabled(False)
            self.s1_bt_6.setEnabled(True)
            self.s4_line_3.setText("串口狀態：已開啟")

    def port_close(self):
        self.recv_timer.stop()
        self.analyze_timer.stop()
        try:
            self.ser.close()
        except:
            pass

        self.s1_bt_5.setEnabled(True)
        self.s1_bt_6.setEnabled(False)
        self.s4_line_3.setText("串口狀態（已關閉)")


    def data_receive(self):
        try:
            num = self.ser.inWaiting()
        except:
            self.port_close()
            return None

        if num > 0:
            self.serial_data = self.ser.readline()
            #self.serial_data = self.ser.read(num)
            self.raw_broswer.insertPlainText(self.serial_data.decode('iso-8859-1'))
            textCursor = self.raw_broswer.textCursor()
            textCursor.movePosition(textCursor.End)
            self.raw_broswer.setTextCursor(textCursor)
        else:
            pass

    def data_send(self):
        MIN_STAMP = 'I'
        MXA_STAMP = 'X'
        END_STAMP = 'H8'

        if self.ser.isOpen():
            input_s = MIN_STAMP + str(self.s2_di_1.value()) + MXA_STAMP + str(self.s2_di_2.value()) + END_STAMP
            #print(input_s)
            input_s = input_s.encode('utf-8')
            self.ser.write(input_s)
        else:
            pass

        self.s2_lb_3.setText("設置最小值：%u 最大值:%u  (km/hrs)"%(self.s2_di_1.value()*2, self.s2_di_2.value()*5))

    def analyze_data(self):
        data = str(self.serial_data)
        list_start = [m.start() for m in re.finditer('@', data)]
        list_end = [m.start() for m in re.finditer('#', data)]
        list_dot = [m.start() for m in re.finditer(',', data)]

        global i_raw_line, x_raw_line

        '''
        '''

        if len(list_start) > 0:
            try:
                self.s4_line_1.display(int(data[list_dot[255] + 2:list_dot[256]])/100)
                self.s4_line_2.display(int(data[list_dot[256]+2:list_end[0]]))
                if 'a'== data[list_dot[255]+1]:
                    self.s4_line_3.setText('物體靠近')
                    self.capture_flag = 1
                elif 'b' == data[list_dot[255] + 1]:
                    self.s4_line_3.setText('物體遠離')
                    self.capture_flag = 2
                else:
                    self.s4_line_3.setText('')

            except:
                return None

            if (self.plot_timer.isActive())&(257==len(list_dot)):
                #print("complex analyze")
                try:
                    i_raw_line[0]=int(data[list_start[0]+1:list_dot[0]])
                    x_raw_line[0]=int(data[list_dot[127]+1:list_dot[128]])

                    for i in range (1, 128):
                        i_raw_line[i] = int(data[list_dot[i]+1:list_dot[i+1]])
                        x_raw_line[i] = int(data[list_dot[i+127] + 1:list_dot[i +127 +1]])

                except:
                    return None

    def open_cam(self):
        #self.monitor_timer.start(100)
        self.s4_line_3.setText("攝影機狀態（已開啟)")
        self.bt_cam_start.setEnabled(False)
        self.bt_cam_stop.setEnabled(True)

    def stop_cam(self):
        #self.monitor_timer.stop()
        self.s4_line_3.setText("攝影機狀態（已關閉)")
        self.bt_cam_stop.setEnabled(False)
        self.bt_cam_start.setEnabled(True)

    def start_draw(self):
        self.plot_timer.start(300)
        self.s4_line_3.setText("raw data plot start")
        self.bt_start_draw.setEnabled(False)
        self.bt_stop_draw.setEnabled(True)
        self.bt_save_data.setEnabled(False)

    def pause_draw(self):
        self.plot_timer.stop()
        self.s4_line_3.setText("raw data plot pause")
        self.bt_start_draw.setEnabled(True)
        self.bt_stop_draw.setEnabled(False)
        self.bt_save_data.setEnabled(True)

    def save_raw_data(self):
        global i_raw_line, x_raw_line
        with open('output.csv','w',newline='') as file:
            writer = csv.writer(file, delimiter=',')
            writer.writerow(i_raw_line)
            writer.writerow(x_raw_line)
    def start_record(self):
        self.speed_record_timer.start(250)

        self.s4_bt_1.setDisabled(True)
        self.s4_bt_2.setDisabled(False)
        self.s4_bt_3.setDisabled(True)

    def stop_record(self):
        self.speed_record_timer.stop()

        self.s4_bt_1.setDisabled(False)
        self.s4_bt_2.setDisabled(True)
        self.s4_bt_3.setDisabled(False)

    def record_speed(self):

        fileName = str(list(self.speedRecordDict.keys())[-1]).replace(":","_")+'.csv'

        csv=  open(fileName, 'a')
        columnTitleRow = "time, speed\n"
        csv.write(columnTitleRow)
        for key in self.speedRecordDict.keys():
            time= key
            speed = self.speedRecordDict[key]
            row = time+","+speed+"\n"
            csv.write(row)

        self.speedRecordDict.clear()

        self.s4_bt_1.setDisabled(False)
        self.s4_bt_2.setDisabled(True)
        self.s4_bt_3.setDisabled(True)

    def update_speed_record_dict(self):
        time = QTime.currentTime()
        self.speedRecordDict.update({time.toString('hh:mm:ss:zzz'): str(self.s4_line_1.value())})






if __name__ == "__main__":
    print("system start")
    app = QApplication(sys.argv)
    ui = MainWindow()
    ui.show()
    sys.exit(app.exec_())